"""
PyQt-Fluent-Widgets
===================
A fluent design widgets library based on PyQt5.

Documentation is available in the docstrings and
online at https://pyqt-fluent-widgets.readthedocs.io.

Examples are available at https://github.com/zhiyiYo/PyQt-Fluent-Widgets/tree/master/examples.

:copyright: (c) 2021 by zhiyiYo.
:license: GPLv3, see LICENSE for more details.
"""

__version__ = "0.6.4"

from .components import *
from .common import *
from ._rc import resource